#!/bin/bash

set -e

# find the path in which the script is located
script=`type -p $0`
case $script in 
  /*) : ;;
  *) script=$PWD/$script;;
esac
dir=`dirname $script`

args="`getopt lb:t:h "$@"`"
set -- $args

defaultboardnum[0]=0
defaultnumboards=1
numboards=0

error()
{
    trap 0
    echo "[FAILURE] $@" >&2

    if [ -z "$loopback_connector" ]; then
	for ((i=0; i<numboards; i++)); do
	    $dir/ze_loopback -0 -b${boardnum[$i]}
	done
    fi
    exit 1
}

usage()
{
   trap 0
    echo "usage: myrige-test.sh [ -l ] [ -t <secs> ] [ -b<rank> ]
    -l : an external loopback connector is used
    -t <secs> : duration of the test (default 4s)
    -b <n> : for machine with multiple NIC, test <n>th NIC
	     -b <n> can be used multiple times to specify multiple boards"
  exit 1
    
}


time=4
loopback_type="(Internal loopback)"

while test $# -ge 1 ; do
 case "$1" in
  -l) loopback_connector=1;shift 1
  loopback_type="(External loopback setup)"
  ;;
  -t) time=$2;shift 2;;
  -b) boardnum[numboards]=$2;((numboards+=1));shift 2;;
  -h) usage;;
  --) shift 1;break;;
  *) echo "Unkown param:$1" >&2;exit 1
 esac
done

params="$@"
if ((numboards == 0)); then
	boardnum[0]=$defaultboardnum
	numboards=defaultnumboards;
fi

#see if a board has been specified more than once
for ((i=1; i<numboards; i++)); do
    for ((j=0; j<i; j++)); do
	if [ "${boardnum[$i]}" = "${boardnum[$j]}" ]; then
	    echo "Board ${boardnum[$i]} specified more than once"
	    usage
	fi
    done
done

trap error 0

load()
{
    rmmod myri10ge 2> /dev/null || :
#     if (cd / && modinfo  myri10ge) | grep -q myri10ge_iprev &&
# 	(cd / && modinfo myri10ge) | grep -q myri10ge_recovery ; then 
#       params="myri10ge_iprev=1 myri10ge_recovery=0"
#       ip_loopback=1
#     fi
    if lsmod | grep mx_driver > /dev/null; then
	error "MX appears to be loaded.  Please unload MX."
    fi

    modprobe myri10ge $params
    # wait for possible udev renaming rules to take effect
    sleep 1
    for ((i=0;i<numboards; i++)); do
    	$dir/myri_info -zb${boardnum[$i]} > /dev/null
    	mac[$i]=`$dir/myri_info -zb${boardnum[$i]} | sed -ne 's/.*MAC=//p'`
    	if test -z "${mac[$i]}" ; then
	    error "Did not find board ${boardnum[$i]}"
    	fi
    	for intf in  `sed -ne 's/^\([^:]*\):.*/\1/p' /proc/net/dev` ; do 
       	    if ifconfig $intf | grep -q -i ${mac[$i]} ; then
		intfs="$intf"
	    fi
	done
	set -- $intfs
	interface[$i]="$1"
	if test -z "${interface[$i]}" ; then
	    error "Cannot find interface for myri10ge NIC instance # ${boardnum[$i]}"
	fi
    done

}

setintf()
{
   for ((i=0; i<numboards; i++)); do 
       mac[$i]=`ifconfig ${interface[$i]} | sed -ne 's/.*HWaddr \(..:..:..:..:..:..\).*/\1/p'`
       test -n "${mac[$i]}" || error "Cannot find Mac for ${interface[$i]}"
       ifconfig ${interface[$i]} 10.108.111.112 netmask 255.255.255.255 mtu 9000
   done
#   route add -host 10.108.111.113 dev $intf
#   arp -i $intf -s 10.108.111.113 $mac
}




load

echo "Testing MYRI-10G NIC(s)"
echo "==========================================================="
echo
#if test -n "$ip_loopback" ; then 
#  $dir/myri_loop_test -d 10.108.111.113 -t $time
#fi

for ((i=0; i<numboards; i++)); do
    if test -z "$loopback_connector" ; then
	$dir/ze_loopback -1 -b${boardnum[$i]}
    else
	$dir/ze_loopback -0 -b${boardnum[$i]}
    fi
done
echo

setintf

for ((i=0; i<numboards; i++)); do
    echo "Board ${boardnum[$i]} is interface ${interface[$i]} with mac ${mac[$i]}"
    if ! touch "/var/tmp/myrige-nic-${mac[$i]}.log"; then
	error "Can't write to file /var/tmp/myrige-nic-${mac[$i]}.log"
    fi
done
echo

#tcpdump -i $intf -n -w /tmp/cap -s 10000 &
sleep 4

for ((i=0; i<numboards; i++)); do
    if ! ifconfig ${interface[$i]} | grep -q RUNNING; then
	error "Board ${boardnum[$i]}/${interface[$i]}/${mac[$i]} link is not up"
    fi
done

#spawn off a job for each board tested
#the assumption is that they will be jobs %1 .. %n
for ((i=0; i<numboards; i++)); do
    $dir/myri_loop_test -m ${mac[$i]} -t $time -i ${interface[$i]} \
	> /var/tmp/myrige-nic-${mac[$i]}.log 2>&1 &
done

#wait for results
for ((i=0; i<numboards; i++)); do
    ((job=i+1))
    if wait %$job; then
	status[$i]="good"
	echo "Board ${boardnum[$i]} exit status: SUCCESS"
    else
	status[$i]="bad"
	echo "Board ${boardnum[$i]} exit status: FAIL"
    fi
    cat /var/tmp/myrige-nic-${mac[$i]}.log
    echo
done

if [ -z "$loopback_connector" ]; then
    for ((i=0; i<numboards; i++)); do
	$dir/ze_loopback -0  -b${boardnum[$i]}
    done
fi

#exit with an error if any board failed.
for ((i=0; i<numboards; i++)); do
    if [ ${status[$i]} = "bad" ]; then
	error "At least one board Failed"
    fi
done

trap 0


